Attribute VB_Name = "MODUL_SPRITE"
Option Explicit

'Nun einige Definitionen aus der Win-API:
Public Declare Function BitBlt Lib "Gdi32" (ByVal _
       NachHdc As Long, ByVal x As Long, ByVal y _
       As Long, ByVal w As Long, ByVal h As Long, _
       ByVal vonHdc As Long, ByVal VonX As Long, _
       ByVal VonY As Long, ByVal Modus As Long) _
       As Long
       
Public Declare Function StretchBlt Lib "Gdi32" (ByVal _
       NachHdc As Long, ByVal x As Long, ByVal y As _
       Long, ByVal w As Long, ByVal h As Long, _
       ByVal vonHdc As Long, ByVal VonX As Long, _
       ByVal VonY As Long, ByVal vonW As Long, _
       ByVal vonH As Long, ByVal Modus As Long) _
       As Long

Public Const BIT_COPY = &HCC0020
Public Const BIT_AND = &H8800C6
Public Const BIT_INVERT = &H660046
Public Const BIT_BLACK = &H42&

'Der Datentyp "pos" nimmt die Koordinaten eines Rechtecks
'in einem Bildfeld und das DeviceHandle des Bildfelds
'auf.

Public Type Pos
  x As Long            'X-Position (linker Rand)
  y As Long            'Y-Position (oberer Rand)
  w As Long            'Breite des Rechtecks
  h As Long            'Hhe des Rechtecks
  hDC As Long          'DeviceContext des Bildfelds
End Type

'Der Datentyp Sprite enthlt die Informationen, die fr einen
'Sprite wichtig sind: Die Position der Maske, die Position der
'Bilddaten und die Position des Sprites auf dem Ausgabe-Bildschirm.
'Letzteres wre eigentlich nicht unbedingt ntigt, da die X- und
'Y-Position immer direkt an SPRITE_DRAW bergeben werden. Jedoch
'Kann man eine bestimmte Ausgangsgre des Sprites festlegen, so
'da er vergrert oder verkleinert dargestellt wird

'Bit und Maske werden mit den SPRITE_SET... Funktionen gesetzt

Public Type Sprite
  Bit As Pos              'Bilddaten
  Maske As Pos            'Maske
  NACH As Pos             'Position, an der der Sprite
                          'angezeigt werden soll
End Type

' Diese Prozedur setzt den Inhalt der Position P, die bergeben
' wird. So kann man zum Beispiel die den Maske-Teil eines Sprites
' ndern:
'
' POS_SET Sprite1.Maske, 10, 10, 100, 100, (Bild1.Hdc)
'
' Diese Funktion wird normal nicht vom Programmierer aufgerufen,
' sondern von anderen Prozeduren.
Public Sub POS_SET(P As Pos, x As Long, y As Long, w As Long, h As Long, PicBox As PictureBox)
  P.x = x
  P.y = y
  P.w = w
  P.h = h
  P.hDC = PicBox.hDC
End Sub

' Diese Prozedur setzt den Inhalt der Position P, die bergeben
' wird. So kann man zum Beispiel die den Maske-Teil eines Sprites
' ndern:
'
' POS_SET_MAX Sprite1.Maske, Bild1
'
' Diese Funktion setzt das Rechteck auf das ganze Bildfeld.
' Der Aufruf kommt also gleich mit:
'
' POS_SET P, 0, 0, Bild1.Width, Bild1.Height, (Bild1.HDC)
Public Sub POS_SET_MAX(P As Pos, PicBox As PictureBox)
  P.x = 0
  P.y = 0
  P.w = PicBox.ScaleWidth
  P.h = PicBox.ScaleHeight
  P.hDC = PicBox.hDC
End Sub

' Sprite zeichenen.
' Der bergebene Sprite wird an der Position X,Y mit den
' ber den Sprite eingestellten Werte (Breite, Hhe etc)
' eingezeichnet
Public Sub SPRITE_DRAW(S As Sprite, x As Long, y As Long, PicBox As PictureBox)
  S.NACH.hDC = PicBox.hDC
  Call StretchBlt(S.NACH.hDC, x, y, S.NACH.w, S.NACH.h, S.Maske.hDC, S.Maske.x, S.Maske.y, S.Maske.w, S.Maske.h, BIT_AND)
  Call StretchBlt(S.NACH.hDC, x, y, S.NACH.w, S.NACH.h, S.Bit.hDC, S.Bit.x, S.Bit.y, S.Bit.w, S.Bit.h, BIT_INVERT)
End Sub

' Sprite festlegen. Der Sprite wird auf die bergebenen Werte
' eingestellt. Diese werden in Form von Daten des Typs POS
' bergeben
Public Sub SPRITE_SET(S As Sprite, Bit As Pos, Maske As Pos)
  S.Bit = Bit
  S.Maske = Maske
  S.NACH.x = 0
  S.NACH.y = 0
  S.NACH.w = Bit.w
  S.NACH.h = Bit.h
End Sub

Public Sub SPRITE_SET_MAX(S As Sprite, Bit As PictureBox, Maske As PictureBox)
  Dim B As Pos, m As Pos, N As Pos

    Call POS_SET_MAX(B, Bit)
    Call POS_SET_MAX(m, Maske)
    
    Call SPRITE_SET(S, B, m)
End Sub

' Erstellt mehrere Sprites als Datenfeld (Sprite(1), Sprite(2)...).
' Diese Funktion bekommt keine einzelnen Werte fr die Sprites
' bergeben sondern erstellt die Sprites an Hand eines Gitters.
'
' Es wird davon ausgegangen, da in den Ressourcen mit dem Handle
' Pb1 und Pb2 Bilder vorliegen, die in einzelne Kstchen aufgeteilt sind,
' wobei jedes Kstchen W-Pixel breit und H-Pixel hoch ist. Dadurch kann
' man sehr leicht eine Vielzahl von Sprites erstellen, ohne Einzelbilder
' zeichnen zu mssen.
Public Sub SPRITE_SET_MESH(Mesh() As Sprite, Start As Long, x As Long, y As Long, w As Long, h As Long, Pb1 As PictureBox, Pb2 As PictureBox)
  Dim xPos As Long, yPos As Long
  Dim iX As Long, iY As Long
  Dim i As Long
  
    yPos = 1
    i = Start

    For iY = 1 To y
      xPos = 1
      For iX = 1 To x
        Call SPRITE_SET_SYNC(Mesh(i), xPos, yPos, w, h, Pb1, Pb2)
        i = i + 1
        xPos = xPos + w + 1
      Next iX
      yPos = yPos + h + 1
    Next iY
    Start = i
End Sub

' Diese Prozedur setzt ebenfalls die Daten fr einen Sprite fest. Hierbei
' mssen jedoch die Positionen der Maske und der Bilddaten bereinstimmen
Public Sub SPRITE_SET_SYNC(S As Sprite, x As Long, y As Long, w As Long, h As Long, Bit As PictureBox, Maske As PictureBox)
  S.Bit.x = x
  S.Bit.y = y
  S.Bit.w = w
  S.Bit.h = h
  S.Bit.hDC = Bit.hDC
  S.Maske = S.Bit
  S.Maske.hDC = Maske.hDC
  
  S.NACH.x = 0
  S.NACH.y = 0
  S.NACH.w = w
  S.NACH.h = h
End Sub

Public Sub SPRITE_DRAW_SOLID(S As Sprite, x As Long, y As Long, PicBox As PictureBox)
  S.NACH.hDC = PicBox.hDC
  Call StretchBlt(S.NACH.hDC, x, y, S.NACH.w, S.NACH.h, _
                  S.Bit.hDC, S.Bit.x, S.Bit.y, S.Bit.w, S.Bit.h, BIT_COPY)
End Sub

